@@ -38,15 +38,22 @@
 
 #include <stdexcept>
 #include <iomanip>
+#include <limits>
 
 #include "Base/GCException.h"
 
 #include <GenApi/ChunkAdapterGEV.h>
 #include <GenApi/ChunkAdapterU3V.h>
 #include <GenApi/ChunkAdapterGeneric.h>
+#include <GenApi/Filestream.h>
 
 #include <rc_genicam_api/pixel_formats.h>
 
+#ifdef _WIN32
+#undef min
+#undef max
+#endif
+
 namespace rcg
 {
 
@@ -308,7 +315,14 @@ bool setEnum(const std::shared_ptr<GenApi::CNodeMapRef> &nodemap, const char *na
 
         if (val != 0)
         {
-          GenApi::IEnumEntry *entry=val->GetEntryByName(value);
+          GenApi::IEnumEntry *entry=0;
+
+          try
+          {
+            entry=val->GetEntryByName(value);
+          }
+          catch (const GENICAM_NAMESPACE::GenericException &)
+          { }
 
           if (entry != 0)
           {
@@ -378,7 +392,7 @@ bool setString(const std::shared_ptr<GenApi::CNodeMapRef> &nodemap, const char *
               }
               else
               {
-                p->SetValue(std::stoi(v));
+                p->SetValue(static_cast<bool>(std::stoi(v)));
               }
             }
             break;
@@ -444,7 +458,14 @@ bool setString(const std::shared_ptr<GenApi::CNodeMapRef> &nodemap, const char *
           case GenApi::intfIEnumeration:
             {
               GenApi::IEnumeration *p=dynamic_cast<GenApi::IEnumeration *>(node);
-              GenApi::IEnumEntry *entry=p->GetEntryByName(value);
+              GenApi::IEnumEntry *entry=0;
+
+              try
+              {
+                entry=p->GetEntryByName(value);
+              }
+              catch (const GENICAM_NAMESPACE::GenericException &)
+              { }
 
               if (entry != 0)
               {
@@ -555,18 +576,29 @@ int64_t getInteger(const std::shared_ptr<GenApi::CNodeMapRef> &nodemap, const ch
     {
       if (GenApi::IsReadable(node))
       {
-        GenApi::IInteger *val=dynamic_cast<GenApi::IInteger *>(node);
-
-        if (val != 0)
+        if (node->GetPrincipalInterfaceType() == GenApi::intfIEnumeration)
         {
-          ret=val->GetValue(false, igncache);
+          GenApi::IEnumeration *p=dynamic_cast<GenApi::IEnumeration *>(node);
+          ret=p->GetCurrentEntry(false, igncache)->GetValue();
 
-          if (vmin != 0) *vmin=val->GetMin();
-          if (vmax != 0) *vmax=val->GetMax();
+          if (vmin != 0) *vmin=ret;
+          if (vmax != 0) *vmax=ret;
         }
-        else if (exception)
+        else
         {
-          throw std::invalid_argument(std::string("Feature not integer: ")+name);
+          GenApi::IInteger *val=dynamic_cast<GenApi::IInteger *>(node);
+
+          if (val != 0)
+          {
+            ret=val->GetValue(false, igncache);
+
+            if (vmin != 0) *vmin=val->GetMin();
+            if (vmax != 0) *vmax=val->GetMax();
+          }
+          else if (exception)
+          {
+            throw std::invalid_argument(std::string("Feature not integer: ")+name);
+          }
         }
       }
       else if (exception)
@@ -867,7 +899,7 @@ std::string getString(const std::shared_ptr<GenApi::CNodeMapRef> &nodemap, const
 void checkFeature(const std::shared_ptr<GenApi::CNodeMapRef> &nodemap, const char *name,
                   const char *value, bool igncache)
 {
-  std::string cvalue=rcg::getString(nodemap, name, true, igncache);
+  std::string cvalue=getString(nodemap, name, true, igncache);
 
   if (cvalue != "" && cvalue != value)
   {
@@ -902,7 +934,7 @@ std::shared_ptr<GenApi::CChunkAdapter> getChunkAdapter(const std::shared_ptr<Gen
 }
 
 std::string getComponetOfPart(const std::shared_ptr<GenApi::CNodeMapRef> &nodemap,
-                              const rcg::Buffer *buffer, uint32_t ipart)
+                              const Buffer *buffer, uint32_t ipart)
 {
   std::string component;
 
@@ -983,4 +1015,112 @@ std::string getComponetOfPart(const std::shared_ptr<GenApi::CNodeMapRef> &nodema
   return component;
 }
 
+std::string loadFile(const std::shared_ptr<GenApi::CNodeMapRef> &nodemap, const char *name,
+                     bool exception)
+{
+  std::string ret;
+
+  // load file in pieces of 512 bytes
+
+  GenApi::FileProtocolAdapter rf;
+  rf.attach(nodemap->_Ptr);
+
+  if (rf.openFile(name, std::ios::in))
+  {
+    size_t length=std::numeric_limits<size_t>::max();
+    try
+    {
+      // limit read operation to file size, if available
+      length=rcg::getInteger(nodemap, "FileSize", 0, 0, true);
+    }
+    catch (const std::exception &)
+    { }
+
+    size_t off=0, n=512;
+    std::vector<char> buffer(512);
+
+    while (n > 0 && length > 0)
+    {
+      n=rf.read(buffer.data(), off, std::min(length, buffer.size()), name);
+
+      if (n == 0)
+      {
+        // workaround for reading last partial block if camera reports failure
+
+        n=rcg::getInteger(nodemap, "FileOperationResult");
+
+        if (n > 0)
+        {
+          n=rf.read(buffer.data(), off, n, name);
+        }
+      }
+
+      if (n > 0)
+      {
+        ret.append(buffer.data(), n);
+      }
+
+      off+=n;
+      length-=n;
+    }
+
+    rf.closeFile(name);
+  }
+  else
+  {
+    if (exception)
+    {
+      throw std::invalid_argument(std::string("Cannot open file for reading: ")+name);
+    }
+  }
+
+  return ret;
+}
+
+bool saveFile(const std::shared_ptr<GenApi::CNodeMapRef> &nodemap, const char *name,
+  const std::string &data, bool exception)
+{
+  bool ret=true;
+
+  // store in pieces of 512 bytes
+
+  GenApi::FileProtocolAdapter rf;
+  rf.attach(nodemap->_Ptr);
+
+  if (rf.openFile(name, std::ios::out))
+  {
+    size_t off=0, n=512;
+    while (n > 0)
+    {
+      n=rf.write(data.c_str()+off, off, std::min(static_cast<size_t>(512), data.size()-off), name);
+      off+=n;
+    }
+
+    rf.closeFile(name);
+
+    if (off != data.size())
+    {
+      if (exception)
+      {
+        std::ostringstream out;
+        out << "Error: Can only write " << off << " of " << data.size() << " bytes";
+        throw std::invalid_argument(out.str());
+      }
+
+      ret=false;
+    }
+  }
+  else
+  {
+    if (exception)
+    {
+      throw std::invalid_argument(std::string("Cannot open file for reading: ")+name);
+    }
+
+    ret=false;
+  }
+
+  return ret;
+}
+
 }
